<?php

namespace Koneko\SatCatalogs\Models;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsTo;

class ClaveUnidadConversion extends Model
{
    protected $table = 'sat_clave_unidad_conversiones';

    protected $fillable = [
        'clave_unidad_origen_id',
        'clave_unidad_destino_id',
        'conversion_factor',
    ];

    protected $casts = [
        'conversion_factor' => 'decimal:8',
    ];

    /**
     * Relación con la unidad de medida origen.
     */
    public function unidadOrigen(): BelongsTo
    {
        return $this->belongsTo(ClaveUnidad::class, 'clave_unidad_origen_id');
    }

    /**
     * Relación con la unidad de medida destino.
     */
    public function unidadDestino(): BelongsTo
    {
        return $this->belongsTo(ClaveUnidad::class, 'clave_unidad_destino_id');
    }

    /**
     * Método para calcular la conversión inversa si no existe en la base de datos.
     */
    public function getInverseConversionFactorAttribute()
    {
        return $this->conversion_factor != 0 ? (1 / $this->conversion_factor) : null;
    }
}
